/* -*- mode: c++; c-basic-offset: 3 -*-
 *
 * PANTHEON Project
 *
 * GREYC IMAGE
 * 6 Boulevard Maréchal Juin
 * F-14050 Caen Cedex France
 *
 * This file is free software. You can use it, distribute it
 * and/or modify it. However, the entire risk to the quality
 * and performance of this program is with you.
 *
 * Please email any bugs, comments, and/or additions to this file to:
 * pandore@greyc.ensicaen.fr
 * http://www.greyc.ensicaen.fr/EquipeImage/Pandore/
 */

/**
 * Projet 3A : WAVEATOM transform
 * @authors
 *  Jalal Fadili
 * @date 2007-11-30
 *
 */

/*
 * @file iatomfm2sym_mex.cpp
 * @brief (Pseudo-)inverse transform of wave atom tight frame
 */

#include <math.h>
#include <complex>
#include <iostream>
using namespace std;
#include "complexe.h"
using namespace pandore;
#include "mex.h"
#include "matrix.h"



#ifdef MAIN
#undef MAIN

#include "iatomfm2sym.cpp"
#include "pan2mex.cpp"

#endif


void mexFunction(int nlhs, mxArray* plhs[], int nrhs, const mxArray* prhs[])
{
  Img2dsf ims;
  Img2dsf imd;
  Float* input;
  Float* output;
  int m, n;
  char* pat;
  
  if(nrhs!=2)
	 mexErrMsgTxt("2 inputs required");
  if(nlhs!=1)
	 mexErrMsgTxt("1 output required");
  input	= (Float*)mxGetData(prhs[0]);
  m 	= mxGetM(prhs[0]); 
  n 	= mxGetN(prhs[0]);
  pat	= mxArrayToString(prhs[1]);
  
  ims.New(m,n);
  mex2pan(input, m, n, ims); 
  //ims.New(m,n);
  //memcpy(ims.Vector(),input,m*n*sizeof(Float));
  Iatomfm2Sym(ims, imd, *pat);
  plhs[0] = mxCreateNumericMatrix(imd.Height(), imd.Width(), mxSINGLE_CLASS, mxREAL);
  output = (Float*)mxGetData(plhs[0]);
  //memcpy(output,imd.Vector(),imd.VectorSize()*sizeof(Float));
  pan2mex(imd,output);
  
  ims.Delete();
  imd.Delete();
  
  return;
}
