/* -*- mode: c++; c-basic-offset: 3 -*-
 *
 * PANTHEON Project
 *
 * GREYC IMAGE
 * 6 Boulevard Maréchal Juin
 * F-14050 Caen Cedex France
 *
 * This file is free software. You can use it, distribute it
 * and/or modify it. However, the entire risk to the quality
 * and performance of this program is with you.
 *
 * Please email any bugs, comments, and/or additions to this file to:
 * pandore@greyc.ensicaen.fr
 * http://www.greyc.ensicaen.fr/EquipeImage/Pandore/
 */

/**
 * @author Jalal Fadili
 * @date 2008-02-15
 *
 */

/*
 * @file fatomundec2sym.cpp
 * @brief Forward undecimated wave atom transform
 */

#include <math.h>
#include <complex>
#include <iostream>
using namespace std;
#include "complexe.h"
using namespace pandore;



#ifdef MAIN
#undef MAIN

#include "wutils.cpp"
namespace freq_pat{
#include "freq_pat.cpp"
}
namespace kf_lf{
#include "kf_lf.cpp"
}
namespace kf_rt{
#include "kf_rt.cpp"
}
namespace fftw{
#include "fftw.cpp"
#include "fftw1d.cpp"
}
namespace arithmetic{
#include "arithmetic/pmultcst.cpp"
#include "arithmetic/pdivcst.cpp"
}
namespace Utility{
#include "utility/pinsertsubimage.cpp"
}
#define MAIN
#else
#include "wutils.cpp"
namespace freq_pat{
#include "freq_pat.cpp"
}
namespace kf_lf{
#include "kf_lf.cpp"
}
namespace kf_rt{
#include "kf_rt.cpp"
}
namespace fftw{
#include "fftw.cpp"
}
namespace arithmetic{
#include "arithmetic/pmultcst.cpp"
#include "arithmetic/pdivcst.cpp"
}
namespace Utility{
#include "utility/pinsertsubimage.cpp"
}
#endif


// Fonction retournant l'image des parties reelles d'une image complexe
Img2dsf real(const Img2dC& c){

  Img2dsf res(c.Size());

  for (int i=0; i<c.Height(); i++)
  for (int j=0; j<c.Width(); j++)
    res[i][j] = real(c[i][j]);
  return res;
}

// Fonction retournant l'image des parties reelles d'une image complexe
Img2dsf imag(const Img2dC& c){
  Img2dsf res(c.Size());

  for (int i=0; i<c.Height(); i++)
  for (int j=0; j<c.Width(); j++)
    res[i][j] = imag(c[i][j]);
  return res;
}

// Fonction retournant l'image du module d'une image complexe
Img2dsf mag(const Img2dC& c){
  Img2dsf res(c.Size());

  for (int i=0; i<c.Height(); i++)
  for (int j=0; j<c.Width(); j++)
    res[i][j] = sqrt(real(c[i][j])*real(c[i][j])+imag(c[i][j])*imag(c[i][j]));
  return res;
}

// Fonction construisant une image complexe a partir de 2 images reelles
Img2dC join (const Img2dsf& re, const Img2dsf& im){
  Img2dC res( re.Size());

  for (int k=0; k<re.Height(); k++)
  for (int l=0; l<re.Width(); l++)
    res[k][l] = Complexe(re[k][l], im[k][l]);
  return res;
}

/**
 * Function Fatomundec2Sym - dorward undecimated wave atom transform
 *
 * input x:    the input N-by-N image
 * output c:   image of undecimated wave atom coefficients
 *
 * @param pat : type of frequency partition which satsifies parabolic scaling relationship
 *         equal to 'p' or 'q'
 *   tp:   1-by-2, tp(1) and tp(2) denote the type of transform to use in x and y dimension respectively
 *         1: standard version
 *         -1: Hilbert transformed version
 *         [default set to [1 1]]
 */
Errc Fatomundec2Sym(Img2dsf& x, Img3dsf& c, char pat, Short tp[2]) {
   float B;
   float Ict, Ifm, Ito, Jct, Jfm, Jto, IdxSize, JdxSize;
   int A, D, N, H;

   // 1D
   Img1dC psi_Ir, psi_Il, psi_Jr, psi_Jl;

   Img1dsf omega;

   //2D
   Img2dsf empty;

   Img2dsf res_re;
   Img2dsf res_im;

   Img2dsf ires_re;
   Img2dsf ires_im;
   Img2dsf ires_tr;

   Img2dsf imd_re;
   Img2dsf imd_im;

   Img2dC res, xft;

   //Collection lst;
   Img1duc* lst;
   Ulong nb_scales; // the number of scales
   Ulong nw; // the packet index
   Ulong nbleaves; // the total number of packets
   Ulong dI,dJ,dtr_I,dtr_J,J;
 

   /******************************/
   N = x.Width();
   J = nextpow2(N);
   H = N/2;
   freq_pat::Freq_pat(lst,H,pat);
   omega.New(N);
   /******************************/

   empty.New(N,N);
   empty = 0; // empty image for the fftw

   imd_re.New(N,N);
   imd_im.New(N,N);
   xft.New(N,N);
   fftw::FFTWNORM(x,empty,imd_re,imd_im,1);

   xft = join (imd_re, imd_im);
 
   // Get the number of scales
   nb_scales = (pat=='p' ? (int) ceil(log2((double)H)/2)+1:(int) floor(log2((double)H)/2)+1);

   nbleaves = 0;
   for(unsigned int s=1; s<nb_scales+1; s++){
   nw = lst[s-1].Width();
   for(unsigned int i=0; i<nw; i++)
   for(unsigned int j=0; j<nw; j++)
   	if(lst[s-1][i]!=0 || lst[s-1][j]!=0)  nbleaves++;
   }
      
   c.New(nbleaves,N,N);
   res.New(N,N);
   res_re.New(N,N);
   res_im.New(N,N);
   ires_re.New(N,N);
   ires_im.New(N,N);
   
   nbleaves = 0;   			
   for(unsigned int s=1; s<nb_scales+1; s++){
   
      nw = lst[s-1].Width();
      for(int iw = -N/2;iw < N/2;iw++)
      	omega[iw+N/2] = (1<<(J-s))*2*M_PI*((float)iw)/N;

      // wave atoms coefficients computation
      for(unsigned int i=0; i<nw; i++){
	 for(unsigned int j=0; j<nw; j++){

	    if(lst[s-1][i]!=0 || lst[s-1][j]!=0){
		     
		     if(s==nb_scales && i==nw-1){
		     	psi_Ir = kf_rt::kf_rt(omega, 0, tp[0]);
		        psi_Il = kf_lf::kf_lf(omega, 0, tp[0]);
			dI     = 0;
		     }
		     else{
		       psi_Ir = kf_rt::kf_rt(omega, i, tp[0]);
		       psi_Il = kf_lf::kf_lf(omega, i, tp[0]);
		       dI     = N >> 1;
		     }
		     
		     if(s==nb_scales && j==nw-1){
		     	psi_Jr = kf_rt::kf_rt(omega, 0, tp[1]);
		        psi_Jl = kf_lf::kf_lf(omega, 0, tp[1]);
			dJ     = 0;
		     }
		     else{
		       psi_Jr = kf_rt::kf_rt(omega, j, tp[1]);
		       psi_Jl = kf_lf::kf_lf(omega, j, tp[1]);
		       dJ     = N >> 1;
		     }
		     
		     ires_tr.New(N,N,&c[nbleaves][0][0]);
		      
		     for (unsigned int k=0; k<N; k++)
		     for (unsigned int l=0; l<N; l++)
			res[k][l] = conj((psi_Ir[(k+dI)%N]+psi_Il[(k+dI)%N]) * (psi_Jr[(l+dJ)%N]+psi_Jl[(l+dJ)%N])) * xft[k][l];
			
		     res_re = real(res);
		     res_im = imag(res);
		     
		     fftw::FFTWNORM(res_re,res_im,ires_re,ires_im,-1); // (-1) -> ifft
		     arithmetic::PMultCst(ires_re,ires_re,(float)(1<<(J-s)));
		     
		     dtr_I = i==0 ? 0:1<<(J-s-1);
		     dtr_J = j==0 ? 0:1<<(J-s-1);
		     
		     for (unsigned int k=0; k<N; k++)
		     for (unsigned int l=0; l<N; l++)
			ires_tr[k][l] = ires_re[(int)mod(k-dtr_I,N)][(int)mod(l-dtr_J,N)];
	
	       ires_tr.Delete();
	       
	       nbleaves++;

	    }
	 } 
      } // end of the nw loop.
      
   }
   
   res.Delete();
   res_re.Delete();
   res_im.Delete();
   ires_re.Delete();
   ires_im.Delete();
   
   return SUCCESS;
}


#ifdef MAIN
#define USAGE   "usage: %s pat(p or q) tp[0](1 or -1) tp[1](1 or -1) [-m mask] [im_in|-] [im_out|-]"
#define PARC    3
#define FINC    1
#define FOUTC   1
#define MASK    0

int main(int argc,char* argv[]) {
   Errc result;			// The result code of the execution.
   Pobject *mask;		// The mask.
   Pobject *objin[FINC+1];	// The input objects;
   Pobject *objs[FINC+1];	// The source objects masked by mask.
   Pobject *objout[FOUTC+1];	// The output object unmasked by mask.
   Pobject *objd[FOUTC+1];	// The result object.
   char *parv[PARC+1];		// The input parameters.
  
   ReadArgs(argc,argv,PARC,FINC,FOUTC,&mask,objin,objs,objout,objd,parv,USAGE,MASK);

   switch(objs[0]->Type()){
   case Po_Img2dsf :{
      Img2dsf* const ims=(Img2dsf*)objs[0];
      objd[0] = new Img3dsf();
      Img3dsf* imd = (Img3dsf*) objd[0];

      Short tp[2];
      tp[0] = atoi(parv[1]);
      tp[1] = atoi(parv[2]);

      result=Fatomundec2Sym(*ims, *imd, parv[0][0], tp);
   }break;
   default :
      PrintErrorFormat(objin,FINC);
      result =FAILURE;
   }
  
   if (result)
      WriteArgs(argc,argv,PARC,FINC,FOUTC,&mask,objin,objs,objout,objd,MASK);
   Exit(result);
   return 0;
}

#endif
