/* -*- c-basic-offset: 3 -*-
 *
 * PANDORE (PANTHEON Project)
 *
 * GREYC IMAGE
 * 6 Boulevard Marchal Juin
 * F-14050 Caen Cedex France
 *
 * This file is free software. You can use it, distribute it
 * and/or modify it. However, the entire risk to the quality
 * and performance of this program is with you.
 *
 * Please email any bugs, comments, and/or additions to this file to:
 * pandore@greyc.ensicaen.fr
 * http://www.greyc.ensicaen.fr/EquipeImage/Pandore/
 */

/**
 * @author Jalal Fadili - 2005-12-20
 * With courtesy of Jean-Luc Starck (CEA 1996).
 */

#include <pandore.h>
using namespace pandore;
#include "fitsio.h"

/**
 * @file pfits2pan.cpp
 * Convert FITS image to Pandore image.
 */ 
void  readdataf(fitsstruct *field, float *ptr) {
   readdataf(field->file, field->filename, field->bitpix, field->npix, ptr, (float)field->bscale, (float)field->bzero);
}

Errc PFits2Pan( fitsstruct *FitsHeader, Img1dsf &imd ) {
   readdataf(FitsHeader, (float*)imd.Vector());
   
   return SUCCESS;
}

Errc PFits2Pan( fitsstruct *FitsHeader, Img2dsf &imd ) {
   readdataf(FitsHeader, (float*)imd.Vector());
   
   return SUCCESS;
}

Errc PFits2Pan( fitsstruct *FitsHeader, Img3dsf &imd ) {
   readdataf(FitsHeader, (float*)imd.Vector());
   
   return SUCCESS;
}

Pobject* PFits2Pan( char *filename) {
   fitsstruct header;
   Errc result;
   
   header.filename = strdup(fitsname(filename));
   header.file = fits_file_des_in(filename);
   readimagehead(&header);
   
   if (header.naxis == 0) {
      cerr << "Error: bad NAXIS keywords in " << filename << endl;
      exit(0);
   } else if (header.naxis == 1) {
      Img1dsf* imd=new Img1dsf((header.TabAxis)[0]);
      result=PFits2Pan(&header, *imd);
      fclose(header.file);
      if (result) return imd; else return NULL;
   } else if (header.naxis == 2) {
      Img2dsf* imd=new Img2dsf(header.height, header.width);
      result=PFits2Pan(&header, *imd);
      fclose(header.file);
      if (result) return imd; else return NULL;
   } else if (header.naxis == 3) {
      //printf("X=%d Y=%d Z=%d bitpix=%d\n",(header.TabAxis)[0],(header.TabAxis)[1],(header.TabAxis)[2],header.bitpix);
      Img3dsf* imd=new Img3dsf((header.TabAxis)[2],(header.TabAxis)[0],(header.TabAxis)[1]);
      result=PFits2Pan(&header, *imd);
      fclose(header.file);
      if (result) return imd; else return NULL;
   } else {
      cerr << "Error: bad NAXIS keywords in " << filename << endl;
      cerr << "       NAXIS must between 1 and 3" << endl;
      return NULL;
   }
}

#ifdef MAIN
#define USAGE   "USAGE : %s [im_in|-] [im_out|-]"
#define PARC    0
#define FINC    0
#define FOUTC   1
#define MASK    0

int main( int argc, char *argv[] ) {
   Errc result;              // The result code of the execution.
   Pobject *mask;            // The mask.
   Pobject *objin[FINC+1];   // The input objects;
   Pobject *objs[FINC+1];    // The source objects masked by mask.
   Pobject *objout[FOUTC+1]; // The output object unmasked by mask.
   Pobject *objd[FOUTC+1];   // The result object.
   char* parv[PARC+1];     // The input parameters.
   
   if (argc<2){
      fprintf(stderr,USAGE"\n",argv[0]);
      Exit((Errc)FAILURE);
   }
   if ((argc==2) && (!strcmp(argv[1],"-p"))){
      fprintf(stdout,"%s 0 1 1\n",argv[0]);
      exit(0);
   }
   
   ReadArgs(argc,argv,PARC,FINC,FOUTC,&mask,objin,objs,objout,objd,parv,USAGE,MASK);     
   objd[0]=PFits2Pan( argv[1] );
   if (objd[0]==NULL) {
      PrintErrorFormat(objin,FINC);
      result = FAILURE;
   } else {
      WriteArgs(argc,argv,PARC,FINC+1,FOUTC,&mask,objin,objs,objout,objd,MASK);
   }
   
   Exit(result);
   return 0;
}

#endif
