/* -*- mode: c++; c-basic-offset: 3 -*-
 *
 * PANDORE (PANTHEON Project)
 *
 * GREYC IMAGE
 * 6 Boulevard Marchal Juin
 * F-14050 Caen Cedex France
 *
 * This file is free software. You can use it, distribute it
 * and/or modify it. However, the entire risk to the quality
 * and performance of this program is with you.
 *
 * Please email any bugs, comments, and/or additions to this file to:
 * pandore@greyc.ensicaen.fr
 * http://www.greyc.ensicaen.fr/EquipeImage/Pandore/
 */

/**
 * @author Jalal Fadili - 02-02-2005
 * @author Christophe Glinel - 20-07-2005
 */

/**
 * @file udwt.cpp
 *
 * Inverse Local Discrete Cosine Transform
 * This LDCT is only implemented with 2D
 * gray images.
 *
 * Transforme inverse discrte en cosinus locaux  (ILDCT)
 * La ILDCT implmente ici ne traite que le cas des images
 * 2D de niveaux de gris. 
 */

#include "dctutils.cpp"
 
#ifdef MAIN
#undef MAIN
// We use the IDCT operator.
#include "idct.cpp"
#define MAIN
#endif


/**
 * ILDCT
 * Integer 2D gray image. (Img2duc)
 * Images MUST have their dimensions all powers of 2 -> reshape.
 * The dimensions of the input and output must be the same.
 * Inputs
 *	ims : The input image.
 * 	imd : The output image.
 * 	ww  : The width of the window.
 */
Errc ILDCT(Img2duc &ims,Img2dsf &imd, Ulong ww){
   Long width,height,depth;
   Long x,y,d1,d2;
	
   CheckWindow(imd,ww);
	
   Img2dsf itemps(ww,ww),itempd(ww,ww);

   height = imd.Height();
   width  = imd.Width();
   depth = (Long)floor((Double)MIN(width,height)/(Double)ww);
	
   /* Go through blocks. */
   for( d1 = 0 ; d1 < depth ; d1++)
      for( d2 = 0 ; d2 < depth ; d2++){
	
	 for( x = 0 ; x < (Long)ww ; x++)
	    for( y = 0 ; y < (Long)ww ; y++)
	       /* Temporary storage of the source block. */
	       itemps[x][y] = (Img2dsf::ValueType)ims[d1*ww+x][d2*ww+y]; 
	
	 /* IDCT of a block. */
	 IDCT(itemps,itempd); 
		
	 for( x = 0 ; x < (Long)ww ; x++)
	    for( y = 0 ; y < (Long)ww ; y++)
	       /* Copy the local IDCT of the block to the output. */
	       imd[d1*ww+x][d2*ww+y] = (Img2dsf::ValueType)itempd[x][y]; 
      }
	
   itemps.Delete();
   itempd.Delete();
   return SUCCESS;
}

/**
 * ILDCT
 * Integer 2D gray image. (Img2dsl)
 * Images MUST have their dimensions all powers of 2 -> reshape.
 * The dimensions of the input and output must be the same.
 * Inputs
 *	ims : The input image.
 * 	imd : The output image.
 * 	ww  : The width of the window.
 */
Errc ILDCT(Img2dsl &ims,Img2dsf &imd, Ulong ww){
   Long width,height,depth;
   Long x,y,d1,d2;
	
   CheckWindow(imd,ww);
	
   Img2dsf itemps(ww,ww),itempd(ww,ww);

   height = imd.Height();
   width  = imd.Width();
   depth = (Long)floor((Double)MIN(width,height)/(Double)ww);
	
   /* Go through blocks. */
   for( d1 = 0 ; d1 < depth ; d1++)
      for( d2 = 0 ; d2 < depth ; d2++){
	
	 for( x = 0 ; x < (Long)ww ; x++)
	    for( y = 0 ; y < (Long)ww ; y++)
	       /* Temporary storage of the source block. */
	       itemps[x][y] = (Img2dsf::ValueType)ims[d1*ww+x][d2*ww+y]; 
	
	 /* IDCT of a block. */
	 IDCT(itemps,itempd); 
		
	 for( x = 0 ; x < (Long)ww ; x++)
	    for( y = 0 ; y < (Long)ww ; y++)
	       /* Copy the local IDCT of the block to the output. */
	       imd[d1*ww+x][d2*ww+y] = (Img2dsf::ValueType)itempd[x][y]; 
      }
	
   itemps.Delete();
   itempd.Delete();
   return SUCCESS;
}

/**
 * ILDCT
 * Integer 2D gray image. (Img2dsf)
 * Images MUST have their dimensions all powers of 2 -> reshape.
 * The dimensions of the input and output must be the same.
 * Inputs
 *	ims : The input image.
 * 	imd : The output image.
 * 	ww  : The width of the window.
 */
Errc ILDCT(Img2dsf &ims,Img2dsf &imd, Ulong ww){
   Long width,height,depth;
   Long x,y,d1,d2;
	
   CheckWindow(imd,ww);
	
   Img2dsf itemps(ww,ww),itempd(ww,ww);

   height = imd.Height();
   width  = imd.Width();
   depth = (Long)floor((Double)MIN(width,height)/(Double)ww);
	
   /* Go through blocks. */
   for( d1 = 0 ; d1 < depth ; d1++)
      for( d2 = 0 ; d2 < depth ; d2++){
	
	 for( x = 0 ; x < (Long)ww ; x++)
	    for( y = 0 ; y < (Long)ww ; y++)
	       /* Temporary storage of the source block. */
	       itemps[x][y] = (Img2dsf::ValueType)ims[d1*ww+x][d2*ww+y]; 
	
	 /* IDCT of a block. */
	 IDCT(itemps,itempd); 
		
	 for( x = 0 ; x < (Long)ww ; x++)
	    for( y = 0 ; y < (Long)ww ; y++)
	       /* Copy the local IDCT of the block to the output. */
	       imd[d1*ww+x][d2*ww+y] = (Img2dsf::ValueType)itempd[x][y]; 
      }
	
   itemps.Delete();
   itempd.Delete();
   return SUCCESS;
}

#ifdef MAIN
#define USAGE	"USAGE : %s window_width [im_in|-] [im_out|-]"
#define PARC	1
#define FINC	1
#define FOUTC	1
#define MASK	0
int main( int argc, char *argv[] ) {
   Errc result;              // The result code of the execution.
   Pobject* mask;            // The region map.
   Pobject* objin[FINC+1];   // The input objects;
   Pobject* objs[FINC+1];    // The source objects masked.
   Pobject* objout[FOUTC+1]; // The output object.
   Pobject* objd[FOUTC+1];   // The result object of the execution.
   char* parv[PARC+1];       // The input parameters.

   ReadArgs(argc,argv,PARC,FINC,FOUTC,&mask,objin,objs,objout,objd,parv,USAGE,MASK);
   if (objs[0]->Type() == Po_Img2duc){
      Img2duc * const ims=(Img2duc*)objs[0];
      objd[0] = new Img2dsf(ims->Size());
      Check(*ims);
      Img2dsf * const imd = (Img2dsf*)objd[0];
      result = ILDCT(*ims,*imd,atol(parv[0]));
      goto end;   
   }
else 
   if (objs[0]->Type() == Po_Img2dsl){
      Img2dsl * const ims=(Img2dsl*)objs[0];
      objd[0] = new Img2dsf(ims->Size());
      Check(*ims);
      Img2dsf * const imd = (Img2dsf*)objd[0];
      result = ILDCT(*ims,*imd,atol(parv[0]));
      goto end;   
   }
else 
   if (objs[0]->Type() == Po_Img2dsf){
      Img2dsf * const ims=(Img2dsf*)objs[0];
      objd[0] = new Img2dsf(ims->Size());
      Check(*ims);
      Img2dsf * const imd = (Img2dsf*)objd[0];
      result = ILDCT(*ims,*imd,atol(parv[0]));
      goto end;   
   }
else 
  {
     PrintErrorFormat(objin,FINC);
     result = FAILURE;
  }	

end:
  if (result)
	WriteArgs(argc,argv,PARC,FINC,FOUTC,&mask,objin,objs,objout,objd,MASK);
  Exit(result);
  return 0;
}
#endif
