/* -*- c-basic-offset: 3 -*-
 *
 * PANDORE (PANTHEON Project)
 *
 * GREYC IMAGE
 * 6 Boulevard Marchal Juin
 * F-14050 Caen Cedex France
 *
 * This file is free software. You can use it, distribute it
 * and/or modify it. However, the entire risk to the quality
 * and performance of this program is with you.
 *
 * Please email any bugs, comments, and/or additions to this file to:
 * pandore@greyc.ensicaen.fr
 * http://www.greyc.ensicaen.fr/EquipeImage/Pandore/
 */

/**
 * @author Jalal Fadili - 2005-12-20
 * With courtesy of Jean-Luc Starck (CEA 1996).
 */

#include <pandore.h>
using namespace pandore;
#include "fitsio.h"
#include "time.h"

/**
 * @file ppan2fits.cpp
 * Convert Pandore image to FITS image.
 * @param ims the input Pandore image.
 * @param name the name of the fits file.
 */ 
Errc PPan2Fits( Img1duc &ims, char *name ) {
   fitsstruct header;
   
   Img1dsf buf(ims.Size());
   buf=ims;
   
   header.bitpix = BP_FLOAT;
   header.width =  ims.Width();
   header.naxis = 1;
   header.npix = ims.VectorSize();
   header.TabAxis[0] = ims.Width();
   header.filename = fitsname(name);
   header.file = fits_file_des_out(name);
   time_t intps=time(NULL);
   header.origin=new char[1000];
   sprintf(header.origin,"Pandore converter\nCreated on %s",ctime(&intps));
   writeimagehead(&header);
   writedataf(&header, (float*)(buf.Vector()));

   return SUCCESS;
}

Errc PPan2Fits( Img1dsl &ims, char *name ) {
  fitsstruct header;
   
  Img1dsf buf(ims.Size());
  buf=ims;
  
  header.bitpix = BP_FLOAT;
  header.width =  ims.Width();
  header.naxis = 1;
  header.npix = ims.VectorSize();
  header.TabAxis[0] = ims.Width();
  header.filename = fitsname(name);
  header.file = fits_file_des_out(name);
  time_t intps=time(NULL);
  header.origin=new char[1000];
  sprintf(header.origin,"Pandore converter\nCreated on %s",ctime(&intps));
  writeimagehead(&header);
  writedataf(&header, (float*)(buf.Vector()));

  return SUCCESS;
}

Errc PPan2Fits( Img1dsf &ims, char *name ) {
  fitsstruct header;
   
  header.bitpix = BP_FLOAT;
  header.width =  ims.Width();
  header.naxis = 1;
  header.npix = ims.VectorSize();
  header.TabAxis[0] = ims.Width();
  header.filename = fitsname(name);
  header.file = fits_file_des_out(name);
  time_t intps=time(NULL);
  header.origin=new char[1000];
  sprintf(header.origin,"Pandore converter\nCreated on %s",ctime(&intps));
  writeimagehead(&header);
  writedataf(&header, (float*)(ims.Vector()));
   
  return SUCCESS;
}

Errc PPan2Fits( Img2duc &ims, char *name ) {
   fitsstruct header;
   
   Img2dsf buf(ims.Size());
   buf=ims;
   
   header.bitpix = BP_FLOAT;
   header.width =  ims.Width();
   header.height =  ims.Height();
   header.naxis = 2;
   header.npix = ims.VectorSize();
   header.TabAxis[0] = ims.Width();
   header.TabAxis[1] = ims.Height();
   header.filename = fitsname(name);
   header.file = fits_file_des_out(name);
   time_t intps=time(NULL);
   header.origin=new char[1000];
   sprintf(header.origin,"Pandore converter\nCreated on %s",ctime(&intps));
   writeimagehead(&header);
   writedataf(&header, (float*)(buf.Vector()));
   
   return SUCCESS;
}

Errc PPan2Fits( Img2dsl &ims, char *name ) {
   fitsstruct header;
   
   Img2dsf buf(ims.Size());
   buf=ims;
   
   header.bitpix = BP_FLOAT;
   header.width =  ims.Width();
   header.height =  ims.Height();
   header.naxis = 2;
   header.npix = ims.VectorSize();
   header.TabAxis[0] = ims.Width();
   header.TabAxis[1] = ims.Height();
   header.filename = fitsname(name);
   header.file = fits_file_des_out(name);
   time_t intps=time(NULL);
   header.origin=new char[1000];
   sprintf(header.origin,"Pandore converter\nCreated on %s",ctime(&intps));
   writeimagehead(&header);
   writedataf(&header, (float*)(buf.Vector()));
   
   return SUCCESS;
}

Errc PPan2Fits( Img2dsf &ims, char *name ) {
   fitsstruct header;
   
   header.bitpix = BP_FLOAT;
   header.width =  ims.Width();
   header.height =  ims.Height();
   header.naxis = 2;
   header.npix = ims.VectorSize();
   header.TabAxis[0] = ims.Width();
   header.TabAxis[1] = ims.Height();
   header.filename = fitsname(name);
   header.file = fits_file_des_out(name);
   time_t intps=time(NULL);
   header.origin=new char[1000];
   sprintf(header.origin,"Pandore converter\nCreated on %s",ctime(&intps));
   writeimagehead(&header);
   writedataf(&header, (float*)(ims.Vector()));
   
   return SUCCESS;
}

Errc PPan2Fits( Img3duc &ims, char *name ) {
   fitsstruct header;
   
   Img3dsf buf(ims.Size());
   buf=ims;  
   
   header.bitpix = BP_FLOAT;
   header.width =  ims.Width();
   header.height =  ims.Height();
   header.naxis = 3;
   header.npix = ims.VectorSize();
   header.TabAxis[0] = ims.Width();
   header.TabAxis[1] = ims.Height();
   header.TabAxis[2] = ims.Depth();
   header.filename = fitsname(name);
   header.file = fits_file_des_out(name);
   time_t intps=time(NULL);
   header.origin=new char[1000];
   sprintf(header.origin,"Pandore converter\nCreated on %s",ctime(&intps));
   writeimagehead(&header);
   writedataf(&header, (float*)(buf.Vector()));
   
   return SUCCESS;
}

Errc PPan2Fits( Img3dsl &ims, char *name ) {
  fitsstruct header;
   
  Img3dsf buf(ims.Size());
  buf=ims;

  header.bitpix = BP_FLOAT;
  header.width =  ims.Width();
  header.height =  ims.Height();
  header.naxis = 3;
  header.npix = ims.VectorSize();
  header.TabAxis[0] = ims.Width();
  header.TabAxis[1] = ims.Height();
  header.TabAxis[2] = ims.Depth();
  header.filename = fitsname(name);
  header.file = fits_file_des_out(name);
  time_t intps=time(NULL);
  header.origin=new char[1000];
  sprintf(header.origin,"Pandore converter\nCreated on %s",ctime(&intps));
  writeimagehead(&header);
  writedataf(&header, (float*)(buf.Vector()));
   
  //delete header.origin;
  return SUCCESS;
}

Errc PPan2Fits( Img3dsf &ims, char *name ) {
  fitsstruct header;

  header.bitpix = BP_FLOAT;
  header.width =  ims.Width();
  header.height =  ims.Height();
  header.naxis = 3;
  header.npix = ims.VectorSize();
  header.TabAxis[0] = ims.Width();
  header.TabAxis[1] = ims.Height();
  header.TabAxis[2] = ims.Depth();
  header.filename = fitsname(name);
  header.file = fits_file_des_out(name);
  time_t intps=time(NULL);
  header.origin=new char[1000];
  sprintf(header.origin,"Pandore converter\nCreated on %s",ctime(&intps));
  header.origin = "Pandore converter";       		
  writeimagehead(&header);
  writedataf(&header, (float*)(ims.Vector()));
   
  return SUCCESS;
}

#ifdef MAIN
#define USAGE   "USAGE : %s [im_in|-] [im_out|-]"
#define PARC    0
#define FINC    1
#define FOUTC   1
#define MASK    0

int main( int argc, char *argv[] ) {
   Errc result;              // The result code of the execution.
   Pobject *mask;            // The mask.
   Pobject *objin[FINC+1];   // The input objects;
   Pobject *objs[FINC+1];    // The source objects masked by mask.
   Pobject *objout[FOUTC+1]; // The output object unmasked by mask.
   Pobject *objd[FOUTC+1];   // The result object.
   char* parv[PARC+1];     // The input parameters.
   
   ReadArgs(argc,argv,PARC,FINC,FOUTC,&mask,objin,objs,objout,objd,parv,USAGE,MASK);
   
  if (objs[0]->Type()==Po_Img1duc) {
     Img1duc* const ims=(Img1duc*)objs[0];
     
     if ((argc>2)&&(strcmp(argv[2],"-")))
	result=PPan2Fits(*ims,argv[2]);
     else
	result=PPan2Fits(*ims,NULL);
     goto end;
  }
  if (objs[0]->Type()==Po_Img1dsl) {
     Img1dsl* const ims=(Img1dsl*)objs[0];
     
     if ((argc>2)&&(strcmp(argv[2],"-")))
	result=PPan2Fits(*ims,argv[2]);
     else
	result=PPan2Fits(*ims,NULL);
     goto end;
  }
  if (objs[0]->Type()==Po_Img1dsf) {
     Img1dsf* const ims=(Img1dsf*)objs[0];
     
     if ((argc>2)&&(strcmp(argv[2],"-")))
	result=PPan2Fits(*ims,argv[2]);
     else
	result=PPan2Fits(*ims,NULL);
     goto end;
  }
  if (objs[0]->Type()==Po_Img2duc) {
     Img2duc* const ims=(Img2duc*)objs[0];
     
     if ((argc>2)&&(strcmp(argv[2],"-")))
	result=PPan2Fits(*ims,argv[2]);
     else
	result=PPan2Fits(*ims,NULL);
     goto end;
  }
  if (objs[0]->Type()==Po_Img2dsl) {
     Img2dsl* const ims=(Img2dsl*)objs[0];
     
     if ((argc>2)&&(strcmp(argv[2],"-")))
	result=PPan2Fits(*ims,argv[2]);
     else
	result=PPan2Fits(*ims,NULL);
     goto end;
  }
  if (objs[0]->Type()==Po_Img2dsf) {
     Img2dsf* const ims=(Img2dsf*)objs[0];
     
     if ((argc>2)&&(strcmp(argv[2],"-")))
	result=PPan2Fits(*ims,argv[2]);
     else
	result=PPan2Fits(*ims,NULL);
     goto end;
  }
  if (objs[0]->Type()==Po_Img3duc) {
     Img3duc* const ims=(Img3duc*)objs[0];
     
     if ((argc>2)&&(strcmp(argv[2],"-")))
	result=PPan2Fits(*ims,argv[2]);
     else
	result=PPan2Fits(*ims,NULL);
     goto end;
  }
  if (objs[0]->Type()==Po_Img3dsl) {
     Img3dsl* const ims=( Img3dsl*)objs[0];
     
     if ((argc>2)&&(strcmp(argv[2],"-")))
	result=PPan2Fits(*ims,argv[2]);
     else
	result=PPan2Fits(*ims,NULL);
     goto end;
  }
  if (objs[0]->Type()==Po_Img3dsf) {
     Img3dsf* const ims=(Img3dsf*)objs[0];
     
     if ((argc>2)&&(strcmp(argv[2],"-")))
	result=PPan2Fits(*ims,argv[2]);
     else
	result=PPan2Fits(*ims,NULL);
     goto end;
  }
  {
     PrintErrorFormat(objin,FINC);
     result = FAILURE;
  }		
  
 end:
  WriteArgs(argc,argv,PARC,FINC,FOUTC,&mask,objin,objs,objout,objd,MASK);
  Exit(result);
  return 0;
}

#endif
