/* -*- c-basic-offset: 3; mode: c++ -*-
 *
 * PANDORE (PANTHEON Project)
 *
 * GREYC IMAGE
 * 6 Boulevard Marchal Juin
 * F-14050 Caen Cedex France
 *
 * This file is free software. You can use it, distribute it
 * and/or modify it. However, the entire risk to the quality
 * and performance of this program is with you.
 *
 * Please email any bugs, comments, and/or additions to this file to:
 * pandore@greyc.ensicaen.fr
 * http://www.greyc.ensicaen.fr/EquipeImage/Pandore/
 */

/**
 * @author Jalal FADILI - 2003-03-31
 */

#include <pandore.h>
using namespace pandore;

/**
 * @file pmedianvalue.cpp
 * Calculates the median value of an image.
 */

/**
 * Comparison function.
 */
int compare( const Float *val1, const Float *val2) {
   if(*val1>*val2) return(1);
   else if(*val1<*val2) return(-1);
   else return(0);
}

/**
 * PMedianValue uses qsort: the median value is at the middle position
 * of the pixel array.
 */

Float PMedianValue( const Img1duc &ims ) {
   Img1duc::ValueType *ps=ims.Vector();
   const Img1duc::ValueType *pf=ims.Vector()+ims.VectorSize();
   Float *pt, *data = new Float[ims.VectorSize()];
   
   pt=data;
   for ( ; ps<pf ; ps++ )
      *(pt++)=(Float)*ps;
      
   int n=ims.VectorSize();
   qsort(data,n,sizeof(Float),(int (*)(const void*, const void*))compare);
   
   Float mediane;
   if (n%2) // Odd number of elements.
      mediane=data[n/2];
   else // Even number of elements.
      mediane= (data[n/2-1 ] + data[n/2])/2;
   
   delete[] data;
   return mediane;
}

Float PMedianValue( const Img1dsl &ims ) {
   Img1dsl::ValueType *ps=ims.Vector();
   const Img1dsl::ValueType *pf=ims.Vector()+ims.VectorSize();
   Float *pt, *data = new Float[ims.VectorSize()];
   
   pt=data;
   for ( ; ps<pf ; ps++ )
      *(pt++)=(Float)*ps;
      
   int n=ims.VectorSize();
   qsort(data,n,sizeof(Float),(int (*)(const void*, const void*))compare);
   
   Float mediane;
   if (n%2) // Odd number of elements.
      mediane=data[n/2];
   else // Even number of elements.
      mediane= (data[n/2-1 ] + data[n/2])/2;
   
   delete[] data;
   return mediane;
}

Float PMedianValue( const Img1dsf &ims ) {
   Img1dsf::ValueType *ps=ims.Vector();
   const Img1dsf::ValueType *pf=ims.Vector()+ims.VectorSize();
   Float *pt, *data = new Float[ims.VectorSize()];
   
   pt=data;
   for ( ; ps<pf ; ps++ )
      *(pt++)=(Float)*ps;
      
   int n=ims.VectorSize();
   qsort(data,n,sizeof(Float),(int (*)(const void*, const void*))compare);
   
   Float mediane;
   if (n%2) // Odd number of elements.
      mediane=data[n/2];
   else // Even number of elements.
      mediane= (data[n/2-1 ] + data[n/2])/2;
   
   delete[] data;
   return mediane;
}

Float PMedianValue( const Img2duc &ims ) {
   Img2duc::ValueType *ps=ims.Vector();
   const Img2duc::ValueType *pf=ims.Vector()+ims.VectorSize();
   Float *pt, *data = new Float[ims.VectorSize()];
   
   pt=data;
   for ( ; ps<pf ; ps++ )
      *(pt++)=(Float)*ps;
      
   int n=ims.VectorSize();
   qsort(data,n,sizeof(Float),(int (*)(const void*, const void*))compare);
   
   Float mediane;
   if (n%2) // Odd number of elements.
      mediane=data[n/2];
   else // Even number of elements.
      mediane= (data[n/2-1 ] + data[n/2])/2;
   
   delete[] data;
   return mediane;
}

Float PMedianValue( const Img2dsl &ims ) {
   Img2dsl::ValueType *ps=ims.Vector();
   const Img2dsl::ValueType *pf=ims.Vector()+ims.VectorSize();
   Float *pt, *data = new Float[ims.VectorSize()];
   
   pt=data;
   for ( ; ps<pf ; ps++ )
      *(pt++)=(Float)*ps;
      
   int n=ims.VectorSize();
   qsort(data,n,sizeof(Float),(int (*)(const void*, const void*))compare);
   
   Float mediane;
   if (n%2) // Odd number of elements.
      mediane=data[n/2];
   else // Even number of elements.
      mediane= (data[n/2-1 ] + data[n/2])/2;
   
   delete[] data;
   return mediane;
}

Float PMedianValue( const Img2dsf &ims ) {
   Img2dsf::ValueType *ps=ims.Vector();
   const Img2dsf::ValueType *pf=ims.Vector()+ims.VectorSize();
   Float *pt, *data = new Float[ims.VectorSize()];
   
   pt=data;
   for ( ; ps<pf ; ps++ )
      *(pt++)=(Float)*ps;
      
   int n=ims.VectorSize();
   qsort(data,n,sizeof(Float),(int (*)(const void*, const void*))compare);
   
   Float mediane;
   if (n%2) // Odd number of elements.
      mediane=data[n/2];
   else // Even number of elements.
      mediane= (data[n/2-1 ] + data[n/2])/2;
   
   delete[] data;
   return mediane;
}

Float PMedianValue( const Img3duc &ims ) {
   Img3duc::ValueType *ps=ims.Vector();
   const Img3duc::ValueType *pf=ims.Vector()+ims.VectorSize();
   Float *pt, *data = new Float[ims.VectorSize()];
   
   pt=data;
   for ( ; ps<pf ; ps++ )
      *(pt++)=(Float)*ps;
      
   int n=ims.VectorSize();
   qsort(data,n,sizeof(Float),(int (*)(const void*, const void*))compare);
   
   Float mediane;
   if (n%2) // Odd number of elements.
      mediane=data[n/2];
   else // Even number of elements.
      mediane= (data[n/2-1 ] + data[n/2])/2;
   
   delete[] data;
   return mediane;
}

Float PMedianValue( const Img3dsl &ims ) {
   Img3dsl::ValueType *ps=ims.Vector();
   const Img3dsl::ValueType *pf=ims.Vector()+ims.VectorSize();
   Float *pt, *data = new Float[ims.VectorSize()];
   
   pt=data;
   for ( ; ps<pf ; ps++ )
      *(pt++)=(Float)*ps;
      
   int n=ims.VectorSize();
   qsort(data,n,sizeof(Float),(int (*)(const void*, const void*))compare);
   
   Float mediane;
   if (n%2) // Odd number of elements.
      mediane=data[n/2];
   else // Even number of elements.
      mediane= (data[n/2-1 ] + data[n/2])/2;
   
   delete[] data;
   return mediane;
}

Float PMedianValue( const Img3dsf &ims ) {
   Img3dsf::ValueType *ps=ims.Vector();
   const Img3dsf::ValueType *pf=ims.Vector()+ims.VectorSize();
   Float *pt, *data = new Float[ims.VectorSize()];
   
   pt=data;
   for ( ; ps<pf ; ps++ )
      *(pt++)=(Float)*ps;
      
   int n=ims.VectorSize();
   qsort(data,n,sizeof(Float),(int (*)(const void*, const void*))compare);
   
   Float mediane;
   if (n%2) // Odd number of elements.
      mediane=data[n/2];
   else // Even number of elements.
      mediane= (data[n/2-1 ] + data[n/2])/2;
   
   delete[] data;
   return mediane;
}


#ifdef MAIN
#define USAGE	"usage: %s [im_in|-]"
#define PARC	0
#define FINC	1
#define FOUTC	0
#define MASK	0

int main( int argc, char *argv[] ) {
   Errc result;              // The result code of the execution.
   Pobject* mask;            // The region map.
   Pobject* objin[FINC+1];   // The input objects;
   Pobject* objs[FINC+1];    // The source objects masked.
   Pobject* objout[FOUTC+1]; // The output object.
   Pobject* objd[FOUTC+1];   // The result object of the execution.
   char* parv[PARC+1];       // The input parameters.

   ReadArgs(argc,argv,PARC,FINC,FOUTC,&mask,objin,objs,objout,objd,parv,USAGE,MASK);
   if (objs[0]->Type() == Po_Img1duc) {
      Img1duc* const ims=(Img1duc*)objs[0];

      result=PMedianValue(*ims);
      goto end;
   }
   if (objs[0]->Type() == Po_Img1dsl) {
      Img1dsl* const ims=(Img1dsl*)objs[0];

      result=PMedianValue(*ims);
      goto end;
   }
   if (objs[0]->Type() == Po_Img1dsf) {
      Img1dsf* const ims=(Img1dsf*)objs[0];

      result=PMedianValue(*ims);
      goto end;
   }
   if (objs[0]->Type() == Po_Img2duc) {
      Img2duc* const ims=(Img2duc*)objs[0];

      result=PMedianValue(*ims);
      goto end;
   }
   if (objs[0]->Type() == Po_Img2dsl) {
      Img2dsl* const ims=(Img2dsl*)objs[0];

      result=PMedianValue(*ims);
      goto end;
   }
   if (objs[0]->Type() == Po_Img2dsf) {
      Img2dsf* const ims=(Img2dsf*)objs[0];

      result=PMedianValue(*ims);
      goto end;
   }
   if (objs[0]->Type() == Po_Img3duc) {
      Img3duc* const ims=(Img3duc*)objs[0];

      result=PMedianValue(*ims);
      goto end;
   }
   if (objs[0]->Type() == Po_Img3dsl) {
      Img3dsl* const ims=(Img3dsl*)objs[0];

      result=PMedianValue(*ims);
      goto end;
   }
   if (objs[0]->Type() == Po_Img3dsf) {
      Img3dsf* const ims=(Img3dsf*)objs[0];

      result=PMedianValue(*ims);
      goto end;
   }
  {
     PrintErrorFormat(objin,FINC);
     result = FAILURE;
  }	

end:
  if (result)
	WriteArgs(argc,argv,PARC,FINC,FOUTC,&mask,objin,objs,objout,objd,MASK);
  Exit(result);
  return 0;
}
#endif
